package ip;

public class IPAddressTest {

	private static class FixedIp implements IPAddress {
		@Override
		public String toString() {
			return "192.168.0.1";
		}

		@Override
		public boolean equals(IPAddress other) {
			return other == this;
		}

		@Override
		public int getOctet(int index) {
			switch (index) {
			case 0:
				return 192;
			case 1:
				return 168;
			case 2:
				return 0;
			case 3:
				return 1;
			}

			throw new RuntimeException("Invalid index");
		}

		@Override
		public IPAddress mask(IPAddress mask) {
			return null;
		}

	}

	private static final IPAddress fixed = new FixedIp();

	public static void main(String[] args) {

		boolean succeeded = testAddress(new IPAddressString("192.168.0.1"),
				new IPAddressString("127.0.0.1"));
		succeeded &= testAddress(new IPAddressShort(new short[] { 192, 168, 0,
				1 }), new IPAddressShort(new short[] { 127, 0, 0, 1 }));
		succeeded &= testAddress(new IPAddressInt(-1062731775),
				new IPAddressInt(2130706433));

		IPAddress[] address = { IPAddressFactory.createAddress("192.168.0.1"),
				IPAddressFactory.createAddress(new short[] { 192, 168, 0, 1 }),
				IPAddressFactory.createAddress(-1062731775) };
		IPAddress[] mask = { IPAddressFactory.createAddress("127.0.0.1"),
				IPAddressFactory.createAddress(new short[] { 127, 0, 0, 1 }),
				IPAddressFactory.createAddress(2130706433) };

		for (IPAddress ip : address)
			for (IPAddress msk : mask)
				succeeded &= testAddress(ip, msk);

		if (succeeded)
			System.out.println("IPAddress Passed");
	}

	private static boolean testAddress(IPAddress ip, IPAddress mask) {
		boolean succeeded = true;

		if(ip == null) {
			System.out.println("ip address creation failed");
			return false;
		}
		
		if (!"192.168.0.1".equals(ip.toString())) {
			succeeded = false;
			System.out.println("toString() failed with "
					+ ip.getClass().getName());
		}

		if (ip.getOctet(0) != 192 || ip.getOctet(1) != 168
				|| ip.getOctet(2) != 0 || ip.getOctet(3) != 1) {
			succeeded = false;
			System.out.println("getOctet failed with "
					+ ip.getClass().getName());
		}

		if (!ip.equals(fixed)) {
			succeeded = false;
			System.out.println("equals failed with " + ip.getClass().getName());
		}

		IPAddress masked = ip.mask(mask);
		if ((masked == null)
				|| (masked.getOctet(0) != 64 || masked.getOctet(1) != 0
						|| masked.getOctet(2) != 0 || masked.getOctet(3) != 1)) {
			succeeded = false;
			System.out.println("mask failed with " + ip.getClass().getName());
		}

		return succeeded;
	}
}
