package test;

import interfaces.Image;

import java.io.IOException;

import transform.ClockwiseRotate;
import transform.Crop;
import transform.Denoise;
import transform.Flip;
import transform.Segmentation;

public class ImageTest {
	
	public static void main(String[] args) throws IOException {
		int[][] bitmap = { { 1, 2, 3, 4 }, { 5, 6, 7, 8 } };
		int[][] seg_bitmap = { { 1, 200, 200, 4 }, { 5, 196, 140, 8 } };
		int[][] noise_bitmap = { { 190, 255, 190, 100, 100, 100 },
				{ 190, 190, 190, 100, 0, 100 } };
		Image source = new MemoryImage(bitmap);
		Image seg_source = new MemoryImage(seg_bitmap);
		Image noise_source = new MemoryImage(noise_bitmap);

		Image lena = new GrayscaleImage("resources/lena.bmp");

		GrayscaleImage lena_noise = new GrayscaleImage(
				"resources/lena_with_noise.bmp");

		String[] tests = { "Flip", "Crop", "Segmentation", "Segmentation3",
				"Denoise", "CW_Rotate" };
		Image[] actual = { new Flip(source), new Crop(source, 1, 1, 2, 1),
				new Segmentation(seg_source, 2),
				new Segmentation(seg_source, 3),
				new Denoise(noise_source, 20, 240, 1),
				new ClockwiseRotate(source) };
		int[][][] expected = {
				{ { 5, 6, 7, 8 }, { 1, 2, 3, 4 } },
				{ { 6, 7 } },
				{ { 0, 255, 255, 0 }, { 0, 255, 255, 0 } },
				{ { 0, 255, 255, 0 }, { 0, 255, 127, 0 } },
				{ { 190, 190, 190, 100, 100, 100 },
						{ 190, 190, 190, 100, 100, 100 } },
				{ { 5, 1 }, { 6, 2 }, { 7, 3 }, { 8, 4 } } };

		Image[] actual2 = { new Flip(lena), new Crop(lena, 110, 90, 70, 100),
				new Segmentation(lena, 2), new Segmentation(lena, 3),
				new Denoise(lena_noise, 10, 245, 5), new ClockwiseRotate(lena) };
		String[] expected2 = { "lena_flip.bmp", "lena_crop.bmp",
				"lena_segmentation.bmp", "lena_segmentation3.bmp",
				"lena_denoise.bmp", "lena_cw_rotate.bmp" };

		for (int i = 0; i < tests.length; ++i) {
			boolean succeeded = small_example_test(tests[i], actual[i],
					expected[i]);

			succeeded &= large_example_test(tests[i], actual2[i], expected2[i]);

			if (succeeded) {
				System.out.println(tests[i] + " Passed");
			}
		}

	}

	private static boolean large_example_test(String string, Image actual,
			String filename) throws IOException {
		Image expected = new GrayscaleImage("resources/" + filename);
		if (!areEqual(actual, expected)) {
			System.out.println(string + " second example failed");
			return false;
		}
		return true;
	}

	private static boolean areEqual(Image im1, Image im2) {
		if (im1.getWidth() != im2.getWidth())
			return false;

		if (im1.getHeight() != im2.getHeight())
			return false;

		for (int y = 0; y < im1.getHeight(); ++y)
			for (int x = 0; x < im1.getWidth(); ++x)
				if (im1.getPixel(x, y) != im2.getPixel(x, y))
					return false;
		return true;
	}

	private static boolean small_example_test(String string, Image image,
			int[][] is) {
		if (!areEqual(image, new MemoryImage(is))) {
			System.out.println(string + " first example failed");
			return false;
		}

		return true;
	}
}
